#
# spec file for package rust
#
# Copyright (c) 2017 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via http://bugs.opensuse.org/
#


%global prev_version 1.13.0
%bcond_with bootstrap
Name:           rust
Version:        1.14.0
Release:        1.1
Summary:        A systems programming language
License:        MIT or Apache-2.0
Group:          Development/Languages/Other
Url:            http://www.rust-lang.org
Source0:        https://static.rust-lang.org/dist/rustc-%{version}-src.tar.gz
Source100:      %{name}-rpmlintrc
# PATCH-FIX-OPENSUSE: Set DT_SONAME when building dylibs
Patch1:         add-soname.patch
BuildRequires:  cmake
BuildRequires:  curl
BuildRequires:  gcc-c++
BuildRequires:  python
Recommends:     cargo
#FIXME: currently there's no way to have rustc and rustc-beta installed

Obsoletes:      rustc-1_9
Provides:       rustc = %{version}
Provides:       rustc-stable = %{version} 
Conflicts:      rustc-bootstrap

BuildRoot:      %{_tmppath}/%{name}-%{version}-build
ExclusiveArch:  %{ix86} x86_64 %{arm} aarch64
%if 0%{?suse_version}
BuildRequires:  fdupes
%endif
%ifarch %{arm}
BuildRequires:  rustc-bootstrap <= %{prev_version}
BuildRequires:  rustc-bootstrap >= %{prev_version}
    %else
    %ifarch aarch64
BuildRequires:  rustc-bootstrap <= %{prev_version}
BuildRequires:  rustc-bootstrap >= %{prev_version}
        %else
        %if %{with bootstrap}
BuildRequires:  rustc-bootstrap <= %{prev_version}
BuildRequires:  rustc-bootstrap >= %{prev_version}
            %else
            %if %{without bootstrap}
BuildRequires:  rust <= %{version}
BuildRequires:  rust >= %{prev_version}
            %endif
        %endif
   %endif
%endif

%description
Rust is a systems programming language focused on three goals:
safety, speed, and concurrency. It maintains these goals without
having a garbage collector, making it a useful language for a
number of use cases other languages are not good at: embedding
in other languages, programs with specific space and time
requirements, and writing low-level code, like device drivers
and operating systems. It improves on current languages targeting
this space by having a number of compile-time safety checks
that produce no runtime overhead, while eliminating all
data races. Rust also aims to achieve "zero-cost abstractions",
even though some of these abstractions feel like those of a
high-level language. Even then, Rust still allows precise
control like a low-level language would.

%package doc
Summary:        Rust documentation
Group:          Development/Languages/Other

%description doc
Documentation for the Rust language.

%package gdb
Summary:        Gdb integration for rust binaries
Group:          Development/Languages/Other
Supplements:    packageand(%{name}:gdb)
Provides:       rust-gdb = %{version}
Provides:       rustc:%{_bindir}/rust-gdb

%description gdb
This subpackage provides pretty printers and a wrapper script for
invoking gdb on rust binaries.

%prep
%setup -q -n rustc-%{version}
%patch1 -p1

%build
export CPPFLAGS="%{optflags}" # eliminate complain from RPMlint
# FIXME: you should use the %%configure macro
./configure \
    --enable-local-rust \
    --local-rust-root=%{_prefix} \
    --prefix=%{_prefix} \
    --release-channel=stable
make %{?_smp_mflags}

%install
make %{?_smp_mflags} DESTDIR=%{buildroot} install

# Remove executable permission from HTML documentation
# to prevent RPMLINT errors.
chmod -R -x+X %{buildroot}%{_datadir}/doc/rust/html

# Remove files which mention buildroot to prevent RPMLINT errors.
rm %{buildroot}%{_prefix}/lib/rustlib/manifest-rust*
rm %{buildroot}%{_prefix}/lib/rustlib/install.log

# Remove lockfile to avoid errors.
rm %{buildroot}%{_datadir}/doc/rust/html/.lock

# allow parallel installation of docs
mv %{buildroot}%{_datadir}/doc/rust \
   %{buildroot}%{_datadir}/doc/rust-%{version}

%if 0%{?suse_version}
%fdupes %{buildroot}%{_prefix}
%endif

%post   -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%defattr(-,root,root,-)
%{_bindir}/rustc
%{_bindir}/rustdoc
%{_mandir}/*/rust*
%exclude %{_datadir}/doc/rust
%{_prefix}/lib/*.so
%{_prefix}/lib/rustlib
%exclude %{_prefix}/lib/rustlib%{_sysconfdir}/gdb_load_rust_pretty_printers.py
%exclude %{_prefix}/lib/rustlib%{_sysconfdir}/gdb_rust_pretty_printing.py

%files doc
%defattr(-,root,root)
%{_datadir}/doc/rust-%{version}

%files gdb
%defattr(-,root,root,-)
%{_bindir}/rust-gdb
%{_prefix}/lib/rustlib%{_sysconfdir}/gdb_load_rust_pretty_printers.py
%{_prefix}/lib/rustlib%{_sysconfdir}/gdb_rust_pretty_printing.py

%changelog
* Tue Jan 31 2017 luke.nukem.jones@gmail.com
- Adjust build so that aarch and ARM architectures use bootstrap for
  initial build
* Tue Jan 31 2017 luke.nukem.jones@gmail.com
- Fix provides/conflicts/obsoletes
* Fri Dec 23 2016 luke.nukem.jones@gmail.com
- Remove patch 0001-Fix-armv7-autodetection.patch
    + appears to have been fixed upstream.
- Building armv7hl arch with bootstrap binary since previously
  packaged versions haven't successfully built in the past
* Fri Dec 23 2016 luke.nukem.jones@gmail.com
- Update to version 1.14.0
    + Announcement: https://blog.rust-lang.org/2016/12/22/Rust-1.14.html
    + Details: https://github.com/rust-lang/rust/blob/master/RELEASES.md#version-1140-2016-12-22
- Release highlights:
    + support for RFC 1492. This small addition lets you use `..`
    in more places, for example when destructuring a struct or tuple
    + println!(), with no arguments, prints newline
    + Wrapping impls standard binary and unary operators on
    references, as well as the Sum and Product iterators, making
    references to these types easier to use
    + Implement From<Cow<str>> for String and From<Cow<[T]>> for
    Vec<T>. These implementations make sense, but were not yet added.
    + Expand .zip() specialization to .map() and .cloned() for
    improved performance.
    + Implement RefUnwindSafe for atomic types, as these types are
    “unwind safe,” though that wasn’t obvious at first.
    + Specialize Vec::extend to Vec::extend_from_slice for
    performance gains.
    + Don’t reuse HashMap random seeds. This helps to mitigate one
    type of DDoS attack.
    + The internal memory layout of HashMap is more cache-friendly,
    for significant improvements in some operations
    + Impl Add<{str, Cow<str>}> for Cow<str>. We already support Add
    for other string types, so not having it on Cow is inconsistent.
* Sun Dec 18 2016 luke.nukem.jones@gmail.com
- Update to 1.13.0
* Sun Dec 18 2016 luke.nukem.jones@gmail.com
- Add conflicts to help avoid situations where previous versions
  or rustc-bootstrap may be installed
* Sun Dec 18 2016 luke.nukem.jones@gmail.com
- Update to 1.12.1
- Remove patches:
    + 0003-Disable-embedding-timestamp-information.patch - fixed
    by upstream
    + 0002-Add-armv6l-autodetection.patch - no-longer viable
* Fri Dec 16 2016 luke.nukem.jones@gmail.com
- Revert from v1.13 to v1.11 in preparation for alternative
  packaging.
* Wed Nov  2 2016 matwey.kornilov@gmail.com
- Add 0001-Fix-armv7-autodetection.patch
- Add 0002-Add-armv6l-autodetection.patch
  * fix armv6 and armv7 builds
* Tue Sep 20 2016 kgronlund@suse.com
- Update to 1.11
  + Add support for cdylib crate types
- Remove merged patches:
  * 0001-Fix-misleading-intentation-errors-on-gcc-6.0.patch
  * 0002-Fix-GCC-6-misleading-indentation-error-in-hoedown.patch
- Set DT_SONAME when building dylibs
  * Add add-soname.patch
* Tue Sep  6 2016 kgronlund@suse.com
- Move to package named rust
* Thu Aug  4 2016 mvyskocil@opensuse.org
- Update to 1.10
    + -C panic=abort flag for rustc or equivalent in Cargo.toml
    + new crate type cdylib, embeded library for other languages
    + In addition, a number of performance improvements landed in
    the compiler, and so did a number of usability improvements
    across the documentation, rustdoc itself, and various error
    messages.
    + This is the first release, which is guaranteed to be built
    by previous stable release of rustc
- Packaging:
    + drop bootstrap mode and use rustc 1.9
    + move documentation to versioned directory
* Sun Jul 31 2016 jengelh@inai.de
- Use smp_mflags for parallel building. Avoid sh invocation for
  simple ldconfig calls. Drop archaic %%clean section.
  Drop filler words from summary.
* Tue Jul 26 2016 kgronlund@suse.com
- Rename source package to rustc-1_9 to conform to naming standards.
* Tue Jul 12 2016 mvyskocil@opensuse.org
- Rename source package to rustc-190 to avoid unecessary rebuilds
  of rustc packages on upgrade
* Wed Jul  6 2016 kgronlund@suse.com
- Move stage0 binaries into separate package
- Disable embedding timestamp information
- Add 0003-Disable-embedding-timestamp-information.patch
* Tue Jun 28 2016 kgronlund@suse.com
- Rename package to rustc-stable
- Add rpmlintrc
- Make bootstrapping conditional
* Mon Jun 27 2016 kgronlund@suse.com
- Fix misleading indentation errors on GCC 6.0
- Remove snap2.sh
- Add 0001-Fix-misleading-intentation-errors-on-gcc-6.0.patch
- Add 0002-Fix-GCC-6-misleading-indentation-error-in-hoedown.patch
* Mon May 30 2016 mvyskocil@opensuse.org
- Update to version 1.9.0
    + Stabilization of std::panic
    + Deprecation warnings, #[deprecated] attribute
    + Compile time improvements
    + Rolling out use of specialization
    + Library stabilizations
    About 80 library functions and methods are now stable in 1.
    + http://blog.rust-lang.org/2016/05/26/Rust-1.9.html
* Fri Apr 15 2016 mvyskocil@opensuse.org
- Update to version 1.8.0:
    + Various “operator equals” operators, such as += and -=, are now
    overloadable via various traits.
    + Empty struct declaration can contain cutly braces
    + New (non default) cargo based build system for rustc
    + About 20 library functions and methods are now stable in 1.8
* Fri Mar  4 2016 kgronlund@suse.com
- Update to version 1.7.0:
    + Many stabilized APIs
    + Improved library performance
    + BTreeSet and its iterators, Iter, IntoIter, and Range are covariant over their contained type.
    + LinkedList and its iterators, Iter and IntoIter are covariant over their contained type.
    + str::replace now accepts a Pattern, like other string searching methods.
    + Any is implemented for unsized types.
    + Hash is implemented for Duration.
    + Soundness fixes, may break code. See RFC 1214 for more information.
    + Several bugs in the compiler's visibility calculations were fixed.
    + Parsing "." as a float results in an error instead of 0.
    + Borrows of closure parameters may not outlive the closure.
* Wed Jan 20 2016 i@xuzhao.net
- Update to version 1.6.0:
    + Stabilization of libcore and other library functions
    + Crates.io disallows wildcards
* Fri Oct 30 2015 mvyskocil@opensuse.org
- Update to version 1.4.0:
    + Several changes have been made to fix type soundness and improve the
    behavior of associated types. See RFC 1214. Although we have mostly
    introduced these changes as warnings this release, to become errors next
    release, there are still some scenarios that will see immediate breakage.
    + The str::lines and BufRead::lines iterators treat \r\n as line breaks in
    addition to \n.
    + Loans of 'static lifetime extend to the end of a function.
    + str::parse no longer introduces avoidable rounding error when parsing
    floating point numbers. Together with earlier changes to float
    formatting/output, "round trips" like f.to_string().parse() now preserve
    the value of f exactly.  Additionally, leading plus signs are now accepted.
* Sat Oct 17 2015 mvyskocil@opensuse.org
- Split the gdb support to rust-gdb subpackage
* Fri Sep 18 2015 mvyskocil@opensuse.org
- Update to version 1.3.0:
    + API stabilization, including the new Duration API and
    enhancements to Error and Hash/Hasher.
    + The substring matcher now uses a more efficient algorithm.
    + There were improvements to zero filling that speed up
    Vec::resize and Read::read_to_end.
    + The implementation of Read::read_to_end has been specialized
    for stdin and File, resulting in additional speedups.
    + The PartialEq implementation on slices is now much faster.
- Packaging: renamed source package to rustc to match upstream
* Wed Aug 12 2015 mvyskocil@opensuse.org
- Update to version 1.2.0:
    + An across-the-board improvement to real-world compiler performance.
    Representative crates include hyper (compiles 1.16x faster), html5ever
    (1.62x faster), regex (1.32x faster) and rust-encoding (1.35x faster). You
    can explore some of this performance data at Nick Cameron’s preliminary
    tracking site, using dates 2015-05-15 to 2015-06-25.
    + Parallel codegen is now working, and produces a 33%% speedup when
    bootstrapping on a 4 core machine. Parallel codegen is particularly useful
    for debug builds, since it prevents some optimizations; but it can also be
    used with optimizations as an effective -O1 flag. It can be activated
    by passing -C codegen-units=N to rustc, where N is the desired number
    of threads.
* Mon Jul 27 2015 mvyskocil@opensuse.org
- Update to version 1.1.0:
  + The std::fs module has been expanded to expand the set of
    functionality exposed:
  * DirEntry now supports optimizations like file_type and
    metadata which don't incur a syscall on some platforms.
  * A symlink_metadata function has been added.
  * The fs::Metadata structure now lowers to its OS counterpart,
    providing access to all underlying information.
  + The compiler now contains extended explanations of many errors.
    When an error with an explanation occurs the compiler suggests
    using the --explain flag to read the explanation. Error
    explanations are also available online.
  + Thanks to multiple improvements to type checking, as well as other
    work, the time to bootstrap the compiler decreased by 32%%.
- drop tar_scm service and use source urls
* Fri May 15 2015 kgronlund@suse.com
- Update to version 1.0.0:
  + lint: deny transmuting from immutable to mutable, since it's undefined behavior
  + std: update select internals to not use mutable transmuting
  + std: Remove index notation on slice iterators
  + std: Destabilize io::BufStream
  + Make RwLock::try_write try to obtain a write lock
  + std: Remove addition on vectors for now
  + thread: right now you can't actually set those printers
  + Fix #24872, XSS in docs not found page.
  + Update AUTHORS.txt and RELEASES.md for 1.0
  + std: Mark `mem::forget` as a safe function
  + core: impl AsRef<[u8]> for str
  + collections: impl AsRef<[u8]> for String
  + collections: change bounds of SliceConcatExt implementations to use Borrow instead of AsRef
  + Fix invalid references due to the automated string substitution
  + dropck: must assume `Box<Trait + 'a>` has a destructor of interest.
* Thu May  7 2015 opensuse-packaging@opensuse.org
- Rename binary package to rust
- Add build for i586
- Only run fdupes on SUSE builds
- Changed version format
- Update to version 1.0.0~beta4+git.1430848988.f873dc5:
  + Introduce a `FreeRegionMap` data structure. (#22779)
  + Fix #20616
  + std: Fix inheriting standard handles on windows
  + Fix #24895.
  + Fix zero-normalization of the pos of a `MultiByteChar`.
  + lint for mixing `#[repr(C)]` with an impl of `Drop`.
  + Bump prerelease version to .4
  + Add downcasting to std::error::Error
* Wed May  6 2015 matwey.kornilov@gmail.com
- Format spec file
* Wed Apr 29 2015 opensuse-packaging@opensuse.org
- Update to version 1.0.0beta3+git.1429985089.5241bf9:
  + Update Windows caveats
  + Utilize if..let for get_mut doc-comment examples
  + Indicate keywords are code-like in Fuse::reset_fuse doc comment
  + doc: improve/fix 'let' FAQ
  + Fix broken links in the docs
  + Indicate None is code-like in doc comments
  + Fixed typo in hash_map::Entry documentation
  + Remove an unused import on windows
* Wed Apr 29 2015 kgronlund@suse.com
- Version 1.0.0-beta3.
